#!/usr/bin/env python3
import sys
from PIL import Image
import base64

def extract_lsb(image_path):
    """
    Opens the image and extracts the least significant bit (LSB)
    from each channel (R, G, B, A) in order (scanning in xy order).
    Returns the reconstructed data as bytes.
    """
    # Open the image and convert it to RGBA to ensure there are 4 channels
    im = Image.open(image_path)
    im = im.convert("RGBA")
    pixels = list(im.getdata())

    bits = []
    # For each pixel, extract the LSB from each channel
    for pixel in pixels:
        for channel in pixel:
            bits.append(channel & 1)

    # Group the bits into bytes (8 bits per byte)
    byte_array = []
    for i in range(0, len(bits), 8):
        byte_bits = bits[i:i+8]
        if len(byte_bits) < 8:
            break
        byte_val = 0
        for bit in byte_bits:
            byte_val = (byte_val << 1) | bit
        byte_array.append(byte_val)

    return bytes(byte_array)

def main():
    if len(sys.argv) != 2:
        print(f"Usage: {sys.argv[0]} <image_file>")
        sys.exit(1)
    
    image_path = sys.argv[1]
    extracted = extract_lsb(image_path)
    
    #print("Extracted data (raw):")
    #print(extracted)
    
    # Attempt to decode the extracted string from Base64
    try:
        decoded = base64.b64decode(extracted)
        #print("\nDecoded Base64 data:")
        print("\nFLAG: " + decoded.decode('utf-8', errors='replace'))
    except Exception as e:
        print("\nError during Base64 decoding:", e)

if __name__ == '__main__':
    main()

